/***********************************************************************************
ResolucaoEspectral.java - Simula diferentes resolues espectrais para uma imagem 

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Simula diferentes resolues espectrais para uma imagem original com 256 nveis
    de cinza.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. resoluo espectral (nmero de bits) desejada para a imagem.

Sada:

  Painel com imagem na resoluo fornecida.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.Raster;
import java.awt.image.LookupOp;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.image.ByteLookupTable;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ResolucaoEspectral extends JFrame {

   public static void main(String args[]) {

      // Certifica a passagem da imagem e da resoluo como parmetro
      if (args.length != 2) {
         String msg = "Uso: java -cp . ResolucaoEspectral <arquivo_imagem> <bits>";
         System.out.println(msg);
         System.exit(0);
      }

      // L parmetro de resoluo e certifica por valor vlido
      int resolucaoEspectral = 0;
      try {
         resolucaoEspectral = Integer.parseInt(args[1]);
      } catch (IllegalArgumentException e) {
         System.out.println("Valor de 'bits' invalido: " + args[1]);
         System.exit(0);
      }

      if ((resolucaoEspectral < 1) || (resolucaoEspectral > 7)) {
         String msg = "Valor invalido. 0 < bits < 8";
         System.out.println(msg);
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando o nome do arquivo imagem
      JFrame sr = new ResolucaoEspectral(args[0], resolucaoEspectral);

      // Encerra a aplicao clicando no "close"
      sr.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      sr.setVisible(true);
   }

   public ResolucaoEspectral(String aFile, int numBits) {

	  // Define um objeto BufferedImage para encapsular a imagem
      BufferedImage src = null, dest = null;
      int w, h;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
         src = ImageIO.read(file);
      } catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
      }
      System.out.println("Imagem '" + aFile );

      // Atribui nome ao frame
      setTitle("Simula resolucao espectral: " + file.getName());

      w = src.getWidth();
      h = src.getHeight();

      // Simula resoluo espectral
      dest = quantificaImagem(numBits, src);

      // Define GUI com objetos do Swing
      JLabel view = new JLabel(new ImageIcon(dest));
      getContentPane().add(new JScrollPane(view));

      // Define tamanho do Frame
      setSize(w, h);
   }


   // Resoluo espectral usando LookupOp
   public BufferedImage quantificaImagem(int numBits, BufferedImage src) {
      int nb;
      float escala;
      byte[] dados;

      BufferedImage destImagem = null; 
		  Raster srcR = src.getRaster(); 
      WritableRaster destWR = src.getRaster(); 

		  nb = 8 - numBits;
      escala = 255.0f/(255 >> nb);
      dados = new byte[256];
      
      for (int i = 0; i < 256; i++)
         dados[i] = (byte) Math.round(escala*(i >> nb));

      LookupOp lookup = new LookupOp(new ByteLookupTable(0, dados), null);
	    destWR = lookup.filter(srcR, destWR); 
	    destImagem = new BufferedImage(src.getColorModel(), destWR, false, null); 
      
      return destImagem;
   }     

/*
   // Resoluo espectral sem LookupOp
   public BufferedImage quantificaImagem(int numBits, BufferedImage src) {
      int nb, w, h, tipo, nc;
      float escala;

      w = src.getWidth();
      h = src.getHeight();
      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage destImagem = new BufferedImage(w, h, tipo);
      WritableRaster destWR = destImagem.getRaster();
      Raster srcR = src.getRaster();

      System.out.println("Sem LookupOp");
      nb = 8 - numBits;
      escala = 255.0f/(255 >> nb);
      for (int i = 0; i < h; i++)
         for (int j = 0; j < w; j++) {
             nc = srcR.getSample(j, i, 0);
             destWR.setSample(j, i, 0, Math.round(escala*(nc >> nb)));
         }

      return destImagem;
   }     
*/
}

